@@ -14,9 +14,9 @@
 ## plot.NMixPlugCondDensMarg
 ## *************************************************************
 ##
-plot.NMixPlugCondDensMarg <- function(x, ixcond, imargin, over=FALSE, auto.layout=TRUE, type="l", lwd=1, lty, col, main, xlab, ylab, annot=TRUE, ...)
+plot.NMixPlugCondDensMarg <- function(x, ixcond, imargin, over=FALSE, auto.layout=TRUE, type="l", lwd=1, lty, col, main, xlab, ylab, ylim, annot=TRUE, ...)
 {
   return(plot.NMixPredCondDensMarg(x=x, ixcond=ixcond, imargin=imargin, over=over, auto.layout=auto.layout, type=type,
-                                   lwd=lwd, lty=lty, col=col, main=main, xlab=xlab, ylab=ylab, annot=annot, ...))  
+                                   lwd=lwd, lty=lty, col=col, main=main, xlab=xlab, ylab=ylab, ylim=ylim, annot=annot, ...))  
 }
 
